__version__ = "2.0.28"
__source_location__ = "https://github.com/achim1/muonic/"
__docs_hosted_at__  = "http://achim1.github.io/muonic/"
__manual_hosted_at__ = "http://physik-begreifen-zeuthen.desy.de/angebote/kosmische_teilchen/schuelerexperimente/cosmo_experiment/index_ger.html"
